'use client';

import { useEffect, useRef, useState } from 'react';
import styles from './RunnerGame.module.css';
import { initGame, characters, setSelectedCharacter as setGameCharacter } from './game';

export default function RunnerGame() {
  // Group related state together
  const [gameState, setGameState] = useState({
    score: 0,
    finalScore: 0,
    isGameOver: false,
    isScoreAnimating: false,
    selectedCharacter: 'classic' as keyof typeof characters
  });

  // Refs
  const canvasRef = useRef<HTMLCanvasElement>(null);
  const containerRef = useRef<HTMLDivElement>(null);

  // Memoize observers to prevent recreation
  useEffect(() => {
    if (!canvasRef.current || !containerRef.current) return;

    // Initialize game
    initGame(canvasRef.current, containerRef.current);
    
    // Score observer
    const scoreObserver = new MutationObserver((mutations) => {
      mutations.forEach((mutation) => {
        if (mutation.type === 'childList') {
          const newScore = parseInt(mutation.target.textContent || '0', 10);
          setGameState(prev => ({
            ...prev,
            score: newScore,
            isScoreAnimating: true
          }));
          
          // Reset animation
          setTimeout(() => {
            setGameState(prev => ({
              ...prev,
              isScoreAnimating: false
            }));
          }, 300);
        }
      });
    });

    // Game over observer
    const gameOverObserver = new MutationObserver((mutations) => {
      mutations.forEach((mutation) => {
        if (mutation.type === 'attributes' && mutation.attributeName === 'style') {
          const gameOverScreen = mutation.target as HTMLElement;
          const isHidden = gameOverScreen.style.display === 'none';
          
          if (!isHidden) {
            const finalScoreEl = document.getElementById('finalScore');
            setGameState(prev => ({
              ...prev,
              isGameOver: true,
              finalScore: parseInt(finalScoreEl?.textContent || '0', 10)
            }));
          }
        }
      });
    });

    // Attach observers
    const scoreElement = document.getElementById('scoreEl');
    const gameOverScreen = document.getElementById('gameOver');

    if (scoreElement) {
      scoreObserver.observe(scoreElement, { childList: true });
    }

    if (gameOverScreen) {
      gameOverObserver.observe(gameOverScreen, { attributes: true });
    }

    // Cleanup observers
    return () => {
      scoreObserver.disconnect();
      gameOverObserver.disconnect();
    };
  }, []);

  // Handle game restart
  const handleRestart = () => {
    window.location.reload();
  };

  // Add character preview rendering
  useEffect(() => {
    const previewCanvases = document.querySelectorAll('.characterPreview');
    previewCanvases.forEach((canvas: Element) => {
      if (canvas instanceof HTMLCanvasElement) {
        const ctx = canvas.getContext('2d');
        if (ctx) {
          const character = characters[canvas.parentElement?.dataset.character as keyof typeof characters];
          if (character) {
            // Clear canvas
            ctx.clearRect(0, 0, canvas.width, canvas.height);
            
            // Draw character preview
            ctx.save();
            ctx.translate(canvas.width / 2, canvas.height / 2);
            
            // Add shadow
            ctx.shadowColor = 'rgba(0, 0, 0, 0.2)';
            ctx.shadowBlur = 10;
            ctx.shadowOffsetY = 5;

            // Draw body based on character shape
            switch(character.shape) {
              case 'round':
                // Classic character with gradient
                const gradientClassic = ctx.createRadialGradient(0, 0, 0, 0, 0, 25);
                gradientClassic.addColorStop(0, character.color);
                gradientClassic.addColorStop(1, '#ff4757');
                ctx.fillStyle = gradientClassic;
                ctx.beginPath();
                ctx.arc(0, 0, 25, 0, Math.PI * 2);
                ctx.fill();

                // Add shine effect
                ctx.beginPath();
                ctx.arc(-8, -8, 8, 0, Math.PI * 2);
                ctx.fillStyle = 'rgba(255, 255, 255, 0.3)';
                ctx.fill();
                break;

              case 'diamond':
                // Ninja character with metallic effect
                const gradientNinja = ctx.createLinearGradient(-25, -25, 25, 25);
                gradientNinja.addColorStop(0, '#2d3436');
                gradientNinja.addColorStop(0.5, '#636e72');
                gradientNinja.addColorStop(1, '#2d3436');
                ctx.fillStyle = gradientNinja;
                
                ctx.beginPath();
                ctx.moveTo(0, -30);
                ctx.lineTo(25, 0);
                ctx.lineTo(0, 30);
                ctx.lineTo(-25, 0);
                ctx.closePath();
                ctx.fill();

                // Add ninja band
                ctx.fillStyle = '#e74c3c';
                ctx.fillRect(-30, -5, 60, 5);
                ctx.fillStyle = 'rgba(255, 255, 255, 0.2)';
                ctx.fillRect(-30, -5, 60, 2);
                break;

              case 'square':
                // Robot character with metallic panels
                const gradientRobot = ctx.createLinearGradient(-25, -25, 25, 25);
                gradientRobot.addColorStop(0, '#74b9ff');
                gradientRobot.addColorStop(0.5, '#0984e3');
                gradientRobot.addColorStop(1, '#74b9ff');
                ctx.fillStyle = gradientRobot;
                
                ctx.beginPath();
                ctx.roundRect(-25, -25, 50, 50, 5);
                ctx.fill();

                // Add metallic panels
                ctx.fillStyle = 'rgba(255, 255, 255, 0.2)';
                ctx.fillRect(-20, -20, 15, 5);
                ctx.fillRect(5, -20, 15, 5);
                ctx.fillRect(-20, 15, 40, 5);
                break;

              case 'oval':
                // Alien character with glow effect
                ctx.shadowColor = '#00b894';
                ctx.shadowBlur = 15;
                
                const gradientAlien = ctx.createRadialGradient(0, 0, 0, 0, 0, 30);
                gradientAlien.addColorStop(0, '#00b894');
                gradientAlien.addColorStop(0.6, '#00cec9');
                gradientAlien.addColorStop(1, '#00b894');
                ctx.fillStyle = gradientAlien;
                
                ctx.beginPath();
                ctx.ellipse(0, 0, 25, 30, 0, 0, Math.PI * 2);
                ctx.fill();

                // Add antenna
                ctx.beginPath();
                ctx.moveTo(0, -30);
                ctx.lineTo(0, -40);
                ctx.lineTo(5, -45);
                ctx.strokeStyle = '#00cec9';
                ctx.lineWidth = 2;
                ctx.stroke();
                break;
            }

            // Reset shadow for eyes
            ctx.shadowColor = 'transparent';
            ctx.shadowBlur = 0;
            ctx.shadowOffsetY = 0;

            // Draw eyes with glow effect
            switch(character.shape) {
              case 'round':
                drawEyes(ctx, -8, -5, 6, character.eyeColor);
                drawEyes(ctx, 8, -5, 6, character.eyeColor);
                break;
              case 'diamond':
                drawEyes(ctx, -8, 0, 5, '#fff');
                drawEyes(ctx, 8, 0, 5, '#fff');
                break;
              case 'square':
                // Robot screen/visor
                ctx.fillStyle = '#0984e3';
                ctx.fillRect(-15, -10, 30, 8);
                ctx.fillStyle = 'rgba(255, 255, 255, 0.3)';
                ctx.fillRect(-15, -10, 30, 2);
                break;
              case 'oval':
                // Alien eyes with third eye
                drawEyes(ctx, -8, -5, 5, '#fff');
                drawEyes(ctx, 8, -5, 5, '#fff');
                drawEyes(ctx, 0, 5, 6, '#fff');
                break;
            }
            
            ctx.restore();
          }
        }
      }
    });

    // Helper function to draw glowing eyes
    function drawEyes(ctx: CanvasRenderingContext2D, x: number, y: number, size: number, color: string) {
      ctx.save();
      ctx.shadowColor = color;
      ctx.shadowBlur = 5;
      ctx.fillStyle = color;
      ctx.beginPath();
      ctx.arc(x, y, size, 0, Math.PI * 2);
      ctx.fill();
      
      // Add shine to eyes
      ctx.beginPath();
      ctx.arc(x - size/3, y - size/3, size/3, 0, Math.PI * 2);
      ctx.fillStyle = 'rgba(255, 255, 255, 0.5)';
      ctx.fill();
      ctx.restore();
    }
  }, []);

  const handleCharacterSelect = (character: string) => {
    setGameState(prev => ({
      ...prev,
      selectedCharacter: character as keyof typeof characters
    }));
    
    // Remove selected class from all options
    document.querySelectorAll(`.${styles.characterOption}`).forEach(option => {
      option.classList.remove(styles.selected);
    });
    
    // Add selected class to chosen character
    const selectedOption = document.querySelector(`[data-character="${character}"]`);
    if (selectedOption) {
      selectedOption.classList.add(styles.selected);
    }
    
    // Hide character select and start the game
    const characterSelect = document.getElementById('characterSelect');
    if (characterSelect) {
      characterSelect.classList.add(styles.hidden);
    }
    
    // Update the game's selected character and start the game
    setGameCharacter(character as keyof typeof characters);
  };

  return (
    <div ref={containerRef} className={styles.gameWrapper}>
      <canvas ref={canvasRef} className={styles.gameCanvas}></canvas>
      
      {/* Score Display */}
      <div 
        className={`${styles.score} ${gameState.isScoreAnimating ? styles.scoreUpdate : ''}`}
        aria-live="polite"
      >
        Score: <span id="scoreEl">{gameState.score}</span>
      </div>

      {/* Character Selection */}
      <div id="characterSelect" className={styles.characterSelect}>
        <h2>Choose Your Character</h2>
        <div className={styles.charactersGrid}>
          <div 
            className={`${styles.characterOption} ${gameState.selectedCharacter === 'classic' ? styles.selected : ''}`} 
            data-character="classic"
            onClick={() => handleCharacterSelect('classic')}
          >
            <canvas className="characterPreview" width="80" height="80"></canvas>
            <p>Classic</p>
          </div>
          <div 
            className={`${styles.characterOption} ${gameState.selectedCharacter === 'ninja' ? styles.selected : ''}`} 
            data-character="ninja"
            onClick={() => handleCharacterSelect('ninja')}
          >
            <canvas className="characterPreview" width="80" height="80"></canvas>
            <p>Ninja</p>
          </div>
          <div 
            className={`${styles.characterOption} ${gameState.selectedCharacter === 'robot' ? styles.selected : ''}`} 
            data-character="robot"
            onClick={() => handleCharacterSelect('robot')}
          >
            <canvas className="characterPreview" width="80" height="80"></canvas>
            <p>Robot</p>
          </div>
          <div 
            className={`${styles.characterOption} ${gameState.selectedCharacter === 'alien' ? styles.selected : ''}`} 
            data-character="alien"
            onClick={() => handleCharacterSelect('alien')}
          >
            <canvas className="characterPreview" width="80" height="80"></canvas>
            <p>Alien</p>
          </div>
        </div>
      </div>

      {/* Game Over Screen */}
      <div 
        id="gameOver" 
        className={`${styles.gameOver} ${!gameState.isGameOver ? styles.hidden : ''}`}
        role="dialog"
        aria-label="Game Over"
      >
        <h2>Game Over!</h2>
        <p className={styles.finalScore}>
          Score: <span id="finalScore">{gameState.finalScore}</span>
        </p>
        <button 
          onClick={handleRestart} 
          className={styles.continueBtn}
          aria-label="Restart Game"
        >
          Play Again
        </button>
      </div>
    </div>
  );
} 